#!/usr/bin/env python3
'''
Generates subfolders based on filenames within the input directory
and if -move is used, moves the relevant files into these new directories.
Eg. An input directory contains file1.mkv, file1.xml file2.mkv, file2.xml
This will result in directories called file1 and file2 being created, and
file1.mkv and file1.xml will be moved into the file1 directory, with a similar action
for file2
'''
import os
import sys
import shutil
import argparse

def parse_args():
    '''
    Parse command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Generates subfolders based on filenames within the input directory'
        'and if -move is used, moves the relevant files into these new directories.'
        'Eg. An input directory contains file1.mkv, file1.xml file2.mkv, file2.xml'
        'This will result in directories called file1 and file2 being created, and'
        'file1.mkv and file1.xml will be moved into the file1 directory, with a similar action'
        'for file2'
    )
    parser.add_argument(
        '-i',
        help='full path of input directory', required=True
    )
    parser.add_argument(
        '-move',
        help='Moves files into the newly created subfolders', action='store_true'
    )
    parsed_args = parser.parse_args()
    return parsed_args

def main():
    '''
    Generates subfolders based on filenames within the input directory
    and if -move is used, moves the relevant files into these new directories.
    Eg. An input directory contains file1.mkv, file1.xml file2.mkv, file2.xml
    This will result in directories called file1 and file2 being created, and
    file1.mkv and file1.xml will be moved into the file1 directory, with a similar action
    for file2
    '''
    args = parse_args()
    inputs = args.i
    if not os.path.isdir(inputs):
        print(' - Input must be a directory/folder - exiting!')
        sys.exit()
    for dirs in os.listdir(inputs):
        if dirs[0] != '.':
            full_path = os.path.join(inputs, dirs)
            new_dir, _ = (os.path.splitext(full_path))
            if not os.path.isdir(new_dir):
                os.makedirs(new_dir)
                if os.path.isdir(new_dir):
                    print(' - The following directory has been created: %s' % new_dir)
            else:
                print(' - %s already exists' % new_dir)
            if args.move:
                if new_dir in full_path:
                    if os.path.isfile(full_path):
                        print(' - %s will be moved into %s' % (full_path, new_dir))
                        shutil.move(full_path, new_dir)

if __name__ == '__main__':
    main()


#!/usr/bin/env python

import subprocess
import sys
import os
from glob import glob
import shutil
try:
    from ififuncs import set_environment
    from ififuncs import hashlib_manifest
    from ififuncs import make_mediatrace
    from ififuncs import make_mediainfo
    from ififuncs import generate_log
    from ififuncs import get_mediainfo
except ImportError:
    print('*** ERROR - IFIFUNCS IS MISSING - *** \n'
    'dvsip requires that ififuncs.py is located in the same directory'
    ' as some functions are located in that script -'
    'https://github.com/kieranjol/IFIscripts/blob/master/ififuncs.py')
    sys.exit()

def get_input():
    if len(sys.argv) < 2:
        print ('IFI DV SIP CREATION SCRIPT')
        print ('USAGE: PYTHON dvsip.py FILENAME')
        print ('OR')
        print ('USAGE: PYTHON dvsip.py DirectoryNAME')
        print ('If input is a directory, all files will be processed')
        print ('If input is a file, only that file will be processed')
        sys.exit()
    else:
        # Input, either file or firectory, that we want to process.
        input = sys.argv[1]
        # Store the directory containing the input file/directory.
        wd = os.path.dirname(input)
        # Change current working directory to the value stored as "wd"
        os.chdir(os.path.abspath(wd))
        # Store the actual file/directory name without the full path.
        file_without_path = os.path.basename(input)
        # Check if input is a file.
        # AFAIK, os.path.isfile only works if full path isn't present.
        if os.path.isfile(file_without_path):
            print('single file found')
            video_files = []                       # Create empty list
            video_files.append(file_without_path)  # Add filename to list
        # Check if input is a directory.
        elif os.path.isdir(file_without_path):
            os.chdir(file_without_path)
            video_files = (
                glob('*.mov')
                + glob('*.dv')
                )
        else:
            print("Your input isn't a file or a directory.")
            print("What was it? I'm curious.")
        dv_test = []    
        for test_files in video_files:
            codec =  get_mediainfo(
                'codec', '--inform=Video;%Codec%',
                 test_files
                 ).rstrip()
            if codec.rstrip() != 'DV':
                dv_test.append(test_files)
                print('Non-DV file found, skipping')
        for i in dv_test:
            if i in video_files:
                video_files.remove(i)
        return video_files


def make_sip(video_files):
    for filename in video_files: #loop all files in directory
        filenoext = os.path.splitext(filename)[0]
        # Generate new directory names
        metadata_dir = '%s/metadata' % filenoext
        log_dir = '%s/logs' % filenoext
        data_dir = '%s/objects' % filenoext
        # Actually create the directories.
        os.makedirs(metadata_dir)
        os.makedirs(data_dir)
        os.makedirs(log_dir)
        #Generate filenames for new files.
        inputxml = '%s/%s_mediainfo.xml' % (
            metadata_dir, os.path.basename(filename)
            )
        inputtracexml = '%s/%s_mediatrace.xml' % (
            metadata_dir, os.path.basename(filename)
            )
        fmd5 = '%s/%s.framemd5' % (
            metadata_dir, os.path.basename(filename)
            )
        log = '%s/%s_log.log' %  (log_dir, filename)
        generate_log(log, 'Input = %s' % filename)
        fmd5_logfile = log_dir + '/%s_framemd5.log' % filename
        fmd5_env_dict = set_environment(fmd5_logfile)
        fmd5_command = [
            'ffmpeg',    # Create decoded md5 checksums for every frame
            '-i', filename,
            '-report',
            '-f', 'framemd5', '-an',
            fmd5
            ]
        print(fmd5_command)
        subprocess.call(fmd5_command, env=fmd5_env_dict)
        generate_log(
            log,
            'makeffv1.py Framemd5 generation of output file completed'
            )
        if os.path.basename(sys.argv[0]) == 'makeffv1.py':
            shutil.copy(sys.argv[0], log_dir)
        print('Generating mediainfo xml of input file and saving it in %s') % inputxml
        make_mediainfo(inputxml, 'mediaxmlinput', filename)
        print('Generating mediatrace xml of input file and saving it in %s') % inputtracexml
        make_mediatrace(inputtracexml, 'mediatracexmlinput', filename)
        source_parent_dir = os.path.dirname(os.path.abspath(filename))
        manifest = '%s/%s_manifest.md5' % (source_parent_dir, filenoext)
        if os.path.isfile(filename):
            shutil.move(filename, data_dir)
            generate_log(log, 'dvsip.py DV file moved to %s' % data_dir)
        generate_log(log, 'dvsip.py MD5 manifest started')
        hashlib_manifest(filenoext, manifest, source_parent_dir)
def main():
    video_files = get_input()
    make_sip(video_files)
if __name__ == '__main__':
    main()



#!/usr/bin/env python
'''
This script will create a new UUID
via ififuncs.create_uuid and print to terminal
'''
from ififuncs import create_uuid


def main():
    '''
    Prints a new UUID to the terminal
    '''
    new_uuid = create_uuid()
    print new_uuid

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# git commandline test
import sys
import subprocess
import os
import time
import pdb
from easygui import multenterbox, choicebox, multchoicebox

# Check if input file has been passed to the script

if len(sys.argv) < 2:
    print '\nUSAGE: PYTHON REVTMD.PY FILENAME\n'
    print 'Requires EASYGUI\n'    
    sys.exit()
else:
    # Store the filename for the reVTMD XML sidecar.
    revtmd_xmlfile = sys.argv[1] + 'revtmd.xml'
    
    # Store the filename for the mediainfo sidecar.
    mediaxml = sys.argv[1] + '_mediainfo.xml'

    # Store the filename without extension.
    filename_without_path = os.path.basename(sys.argv[1])

    # Store the current time in ISO8601 format.
    time_date = time.strftime("%Y-%m-%dT%H:%M:%S")
    date = time.strftime("%Y-%m-%d")
    
    # Determine number of audio tracks.
    def get_audio_stream_count():
        audio_stream_count = subprocess.check_output(['ffprobe', 
                                                      '-v', 'error', 
                                                       '-select_streams', 'a', 
                                                       '-show_entries', 'stream=index',
                                                       '-of', 'flat',
                                                       sys.argv[1]]).splitlines()
        return len(audio_stream_count)
    audio_tracks = get_audio_stream_count()
    if audio_tracks > 0:
        sound = "Yes"
    elif audio_tracks == 0:
        sound = "No"

        
    # Begin creating functions for repetitive tasks:
    
    # Create verbose mediainfo xml sidecar for harvesting purposes.                    
    with open(mediaxml, "w+") as fo:
            mediaxmlinput = subprocess.check_output(['mediainfo',
                                '-f',
                                '--language=raw', # Use verbose output.
                                '--output=XML',
                                sys.argv[1] ])       #input filename
            fo.write(mediaxmlinput)
        

    # This function actually adds value to a specified xml element.    
    def add_to_revtmd(element, value, xmlfile):
        subprocess.call(['xml', 'ed', '--inplace', '-N', 'x=http://nwtssite.nwts.nara/schema/', '-u', element, '-v', value, xmlfile])

        # What follows are a lot of functions that can be reused. Titles should be self explanatory.

    def get_mediainfo(var_type, type, filename):
        var_type = subprocess.check_output(['mediainfo', '--Language=raw', '--Full',
                                             type , filename ]).replace('\n', '').replace('\r', '')
        return var_type
        
    # Generate xml elements for coding process history.
    def revtmd_coding_process_history():
        fo.write('<revtmd:codingProcessHistory>\n')
        fo.write('<revtmd:role/>\n')
        fo.write('<revtmd:description/>\n')
        fo.write('<revtmd:manufacturer/>\n')
        fo.write('<revtmd:modelName/>\n')
        fo.write('<revtmd:version/>\n')
        fo.write('<revtmd:serialNumber/>\n')
        fo.write('<revtmd:signal/>\n')
        fo.write('<revtmd:settings/>\n')
        fo.write('<revtmd:settings/>\n')
        fo.write('<revtmd:settings/>\n')
        fo.write('<revtmd:settings/>\n')
        fo.write('<revtmd:settings/>\n')
        fo.write('<revtmd:settings/>\n')
        fo.write('<revtmd:videoEncoding/>\n')
        fo.write('</revtmd:codingProcessHistory>\n')
        
    #pdb.set_trace()
    
    def get_xml_output(xpath, element, source_xml,xml_variable):
        command =  ['xml','sel', '-t', '-m', xpath, '-v', element, source_xml ]
        xml_variable = subprocess.Popen(command, stdout=subprocess.PIPE)
        xml_variable = xml_variable.communicate()[0]
        return xml_variable
            
    def tech_metadata_revtmd():
        add_to_revtmd('//revtmd:duration', duration, revtmd_xmlfile)
        add_to_revtmd('//revtmd:PAR', par, revtmd_xmlfile)
        add_to_revtmd('//revtmd:DAR', dar, revtmd_xmlfile)
        add_to_revtmd('//revtmd:pixelsHorizontal', width, revtmd_xmlfile)
        add_to_revtmd('//revtmd:pixelsVertical', height, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]/revtmd:size', size, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]/revtmd:sampling', chromasubsampling, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]//revtmd:codecID', codec_id, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]//revtmd:scanType', scantype, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]/revtmd:bitsPerSample', bitspersample, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]//revtmd:quality', quality, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]//revtmd:scanOrder', scanorder, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]//revtmd:duration', duration, revtmd_xmlfile)
        add_to_revtmd('//revtmd:track[@id="1" and @type="video"]//revtmd:dataRate', video_bitrate , revtmd_xmlfile)
        
    def ffmpeg_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Transcode', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Transcode to FFv1 in Matroska wrapper', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'ffmpeg', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', '2.8.2', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:videoEncoding', "FFv1", revtmd_xmlfile)
        
    def ffmpeg_v210_machine_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Transcode', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Transcode to v210', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'ffmpeg', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', '???', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:videoEncoding', "v210", revtmd_xmlfile)

    def avid_capture_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Capture Software', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'SDI bitstream capture', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Avid', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Media Composer', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '8.5.0', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:videoEncoding', "Avid 1:1 10-bit", revtmd_xmlfile) #bot sure of 4cc right now, maybe AVup?

    def control_room_capture_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Capture Software', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'AJA', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Control Room', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', 'ABC123', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'ABC123', revtmd_xmlfile)   
    def telecine_mac_pro_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Provides computing environment', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Apple', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Mac Pro', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', 'dunno', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'ABC123', revtmd_xmlfile)

    def telecine_mac_pro_os_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer Operating System', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Provides computing environment operating system', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Apple', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Mavericks', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', 'dunno', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'ABC123', revtmd_xmlfile)
    def win7_hp_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer Operating System', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Windows', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', '7 Professional', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', 'Service pack X', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'ABC123', revtmd_xmlfile)
    def avid_export_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Transcode', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Transcode to v210 in quicktime wrapper', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Avid', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Media Composer', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '8.5.0', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:videoEncoding', "v210", revtmd_xmlfile)
    def bmd_us4k_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Capture Card', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Capture SDI signal', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Blackmagic', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Ultrastudio 4k', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '10.5', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', "SDI", revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '1955795', revtmd_xmlfile)
    def bmd_miniconverter_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Analog to digital converter', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Blackmagic', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Mini-Converter Analog to SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '334080', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', "SDI", revtmd_xmlfile)
    def avid_consolidate_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'File Editing', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Add plate, consolidate multiple clips', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Avid', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Media Composer', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '8.5.0', revtmd_xmlfile)
    def avid_post_processing(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Post Processing', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Avid', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Media Composer', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '8.5.0', revtmd_xmlfile)
                
    '''
    to do - add in bmd a2d, and add aja kona as a capture card.
    '''
    def aja_analog2digital(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Analog to Digital Converter', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', '', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'AJA', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'KONA LHe Plus', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'ABC123', revtmd_xmlfile)
    def aja_kona_capture_ingest1(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Capture Card', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Capture SDI Signal', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'AJA', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'KONA LHe Plus', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '00T59106', revtmd_xmlfile)
    def aja_kona_capture_ingest2(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Capture Card', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Capture SDI Signal', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'AJA', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'KONA LHe Plus', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '00T59109', revtmd_xmlfile)
    def IiyamaMonitor_ingest1(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer Monitor', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', '', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Iiyama', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'ProLite B2480HS/B1', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'DVI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '11183M4504454', revtmd_xmlfile)
    def IiyamaMonitor_ingest2(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer Monitor', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', '', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Iiyama', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'ProLite B2480HS/B1', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'DVI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '11183M4504437', revtmd_xmlfile)
    def IiyamaMonitor1_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer Monitor', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', '', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Iiyama', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'ProLite B2480HS/B1', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'DVI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '1183M4504428', revtmd_xmlfile)
    def IiyamaMonitor2_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer Monitor', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', '', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Iiyama', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'ProLite B2480HS/B1', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'DVI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '1183M4504434', revtmd_xmlfile)
    def bmd_ultrascopes_ingest1(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Quality Assesment', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Digital Scopes', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Blackmagic Design', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Smartscope Duo 4K', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '1969088', revtmd_xmlfile)        
    def bmd_ultrascopes_ingest2(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Digital Scopes', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Blackmagic Design', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Smartscope Duo 4K', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '1968794', revtmd_xmlfile)        
    def bmd_ultrascopes_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Digital Scopes', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Blackmagic Design', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Smartscope Duo 4K', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '1969171', revtmd_xmlfile)        
    def tvlogic_broadcast_ingest1(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Ingest 1 Broadcast Monitor', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'TV Logic', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'LVM-245W', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'LV245N0047', revtmd_xmlfile)        
    def tvlogic_broadcast_ingest2(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Ingest 2 Broadcast Monitor', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'TV Logic', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'LVM-245W', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'LV245N0157', revtmd_xmlfile)    
    def tvlogic_broadcast_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Telecine Broadcast Monitor', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'TV Logic', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'LVM-245W', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'LV245N0048', revtmd_xmlfile)
    def philips_headphones_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Audio Monitoring Headphones', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Philips', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'TBD', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'TBD', revtmd_xmlfile)   
    def maudio_left_speaker_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Quality Assesment', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Left Speaker', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'M-AUDIO', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'BX5D', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '(21)CL1404140131071', revtmd_xmlfile)   
    def maudio_right_speaker_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Quality Assesment', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Right Speaker', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'M-AUDIO', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'BX5D', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '(21)CL1404140131072', revtmd_xmlfile)
    def avid_audio_post_posprocessing(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Audio Post Processing', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Digital Audio Workstation', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Avid', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Pro Tools', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '12.5.1', revtmd_xmlfile)
    def izotope_audio_post_processing(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Audio Post Processing', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Audio Cleaning Software', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'iZotope Inc', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'iZotope Rc5', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '5.01.184', revtmd_xmlfile)
    def audacity_audio_post_processing(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Audio Post Processing', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Digital Audio Workstation', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Audacity', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Audacity', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '2.1.2', revtmd_xmlfile)
    def aeolight_audio_post_processing(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Audio Post Processing', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Extracts audio from combined film optical track', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'University of South Carolina', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'AEO-Light', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '1.0', revtmd_xmlfile)
    def focusrite_audio_monitoring(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Sound Card', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Headphone monitoring', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Focusrite', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Scarlett', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '2i2', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'Stereo', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'S364098253417', revtmd_xmlfile)
    def telecine_mac_mini(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Workstation for audio extraction and cleaning', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Apple', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Mac Mini (Late 2012)', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '10.11.4', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'C07LW0YRDY3H', revtmd_xmlfile)
    def maudio_left_studiophile_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Quality Assesment', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Left Speaker', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'M-AUDIO', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Studiophile AV 40', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '132A0602 109128', revtmd_xmlfile)
    def maudio_right_studiophile_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Quality Assesment', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Right Speaker', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'M-AUDIO', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Studiophile AV 40', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '132A0602 109128', revtmd_xmlfile)
    def beyer_headphones_telecine(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Audio Monitoring Headphones', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Beyer Dynamic', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Dt 100/400 ohms', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '421464', revtmd_xmlfile)
    def telecine_mac_pro_os_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer Operating System', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Mac Mini operating system', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Apple', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'OSX El Capitan', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', '10.11.4', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'ABC123', revtmd_xmlfile)
    def basement_content_agent_pc(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Workstation for transcoding files from Steadyframe 2k scanner', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Hewlett-Packard Company', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Z800 Workstation', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', 'FF825AV', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'CZC2082R1Z', revtmd_xmlfile)
    def basement_content_agent_pc_os_revtmd(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer Operating System', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Content Agent PC operating system', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Microsoft', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Windows', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', 'Windows 7 Professional', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '00371-OEM-8992671-00008', revtmd_xmlfile)
    def basement_2k_scanner(number):
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', 'Workstation for 35mm and 16mm film scanning', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'P+S Technik', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Steadyframe', revtmd_xmlfile)
        add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '601-0101', revtmd_xmlfile)	
    def revtmd_blank_audio_fields():   
        global audio_track_number        
        audio_track_number = 1
        while audio_track_number <= audio_tracks:
            if audio_tracks == 1:
                audio_xpath = "Mediainfo/File/track[@type='Audio']"
            else:
                audio_xpath = "Mediainfo/File/track[@type='Audio' and @typeorder=%s]" % (audio_track_number)
           
            aendian = get_xml_output(audio_xpath,'Codec_SettingsEndianness', mediaxml, 'aendian')
            abitdepth = get_xml_output(audio_xpath,'Resolution', mediaxml, 'abitdepth')
            acodec = get_xml_output(audio_xpath,'Codec', mediaxml, 'acodec')
            channel_count = get_xml_output(audio_xpath,'Channel_s_', mediaxml, 'channel_count')        
            asize = get_xml_output(audio_xpath,'StreamSize', mediaxml, 'asize')
            compression_mode = get_xml_output(audio_xpath,'Compression_Mode', mediaxml, 'compression_mode')
            sample_rate = get_xml_output(audio_xpath,'SamplingRate', mediaxml, 'sample_rate')
            a_duration = get_xml_output(audio_xpath,'Duration_String4', mediaxml, 'a_duration')
            a_codecid = get_xml_output(audio_xpath,'CodecID', mediaxml, 'a_codecid')

            fo.write('<revtmd:track id="%s" type="audio">\n' % audio_track_number)
            fo.write('<revtmd:duration/>\n')
            fo.write('<revtmd:size>%s</revtmd:size>\n' % asize)
            fo.write('<revtmd:codec>\n')
            fo.write('<revtmd:name>%s</revtmd:name>\n' % acodec)
            fo.write('<revtmd:codecID>%s</revtmd:codecID>\n' % a_codecid)
            fo.write('<revtmd:channelCount>%s</revtmd:channelCount>\n' % channel_count)
            fo.write('<revtmd:endianness>%s</revtmd:endianness>\n' % aendian)
            fo.write('<revtmd:quality>%s</revtmd:quality>\n'% compression_mode)
            fo.write('</revtmd:codec>\n')
            fo.write('<revtmd:bitsPerSample>%s</revtmd:bitsPerSample>\n'% abitdepth)
            fo.write('<revtmd:samplingRate>%s</revtmd:samplingRate>\n' % sample_rate) 
            fo.write('<revtmd:bitsPerSample/>\n')
            fo.write('<revtmd:sampling/>\n')
            fo.write('</revtmd:track>\n')
                
            audio_track_number += 1
    
    def prep_data_entry():
        if post_processing != None:
            number2 = 1
            for post in post_processing:
                add_to_revtmd("//revtmd:codingProcessHistory[revtmd:role = 'Post Processing']" + '/revtmd:settings' + '[' + str(number2) + ']' , post, revtmd_xmlfile)
                number2 += 1    
    def capture_interventions_func():           
        if capture_interventions != None:
            number2 = 2
            for interventions in capture_interventions:
                add_to_revtmd('//revtmd:codingProcessHistory[2]' + '/revtmd:settings' + '[' + str(number2) + ']' , interventions, revtmd_xmlfile)
                number2 += 1    
            for settings in flashtransfer_settings:
                add_to_revtmd('//revtmd:codingProcessHistory[2]' + '/revtmd:settings' + '[' + str(number2) + ']' , settings, revtmd_xmlfile)
                number2 += 1    
        else:
            print 'all is well'

    # Begin Interview using Easygui.
    msg ="Which Workflow?"
    title = "Workflows"
    choices = ["Telecine One Light", "Scanning", "Audio Extraction", "bestlight", "Telecine Grade", "Tape Ingest 1", "Tape Ingest 2", "Tape Edit Suite 1", "Tape Edit Suite 2"]
    workflow = choicebox(msg, title, choices)

    # Forking path in order to get more accurate info depending on workflow
    if workflow not in ("Telecine One Light", "bestlight", "Telecine Grade", "Audio Extraction", "Scanning"):
        no_of_emptyfields = 16 #temp, this will be a variable.
        msg ="Tape Deck?"
        title = "Pick a name yo!"
        choices = ["DVW-500P", "MiniDV-Something", "UVW-1400AP", "DVW-510P", "J-30", "J-H3", "UVW-1200P", "Unknown"]
        deck = choicebox(msg, title, choices)
        print deck
        if deck == "DVW-A500P":
            def deck_func(number):
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Playback', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Sony', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'DVW-A500P', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '10317', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:settings[1]', 'Timecode = Auto', revtmd_xmlfile)  
        if deck == "DVW-510P":
            def deck_func(number):
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Playback', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Sony', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'DVW-A510p', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '11414', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:settings[1]', 'Timecode = Auto', revtmd_xmlfile)   
        elif deck == "J-30":
            def deck_func(number):
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Playback', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Sony', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'J-30', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'ABC123', revtmd_xmlfile)
        elif deck == "J-H3":
            def deck_func(number):
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Playback', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Sony', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'J-H3', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '11482', revtmd_xmlfile)    
        elif deck == "UVW-1400AP":
            def deck_func(number):
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Playback', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Sony', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'UVW-1400AP', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'Component', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '13697', revtmd_xmlfile) 
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:settings[1]', 'Component out = Y-R,B', revtmd_xmlfile)   
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:settings[2]', 'Timecode = LTC', revtmd_xmlfile)    
                  
        elif deck == "UVW-1200P":
            def deck_func(number):
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Playback', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Sony', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'UVW-1200P', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'Component', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', '13697', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:settings[1]', 'Component out = Y-R,B', revtmd_xmlfile)   
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:settings[2]', 'Timecode = LTC', revtmd_xmlfile)                  
    if workflow == "Tape Ingest 1":
        def workstation(number):
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer', revtmd_xmlfile)
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Hewlett Packard', revtmd_xmlfile)
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Z420 Workstation', revtmd_xmlfile)
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', 'ABC123', revtmd_xmlfile)
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'CZC4310HNZ', revtmd_xmlfile)
        no_of_emptyfields = 6
    elif workflow == "Tape Ingest 2":
        def workstation(number):
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Host Computer', revtmd_xmlfile)
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'Hewlett Packard', revtmd_xmlfile)
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Z420 Workstation', revtmd_xmlfile)
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:version', 'ABC123', revtmd_xmlfile)
            add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:serialNumber', 'CZC4310HP8', revtmd_xmlfile)
    # Currently unused, but I'll get around to it :[
    else:
        no_of_emptyfields = 18
        if workflow == "Telecine Grade":
            msg = "Interventions post capture"
            title = "Interventions post capture?"
            fieldNames = ["Colour Alterations","Exposure Alterations","Sound Alterations"]
            grade_interventions = []  # we start with blanks for the values
            grade_interventions = multenterbox(msg,title, fieldNames)
        
        else:
            if workflow == "Scanning":
                scanner = 'Scanner'
            else:    
                msg = "Capture Frame Rate"
                title = "Capture Frame Rate"
                fieldNames = ["15","16","18","20","22","24","25", "Multiple frame rates"]
                capture_frame_rate = []  # we start with blanks for the values
                capture_frame_rate = choicebox(msg,title, fieldNames)
                fps_string = 'Captured at %s fps' % capture_frame_rate
            
                msg ="Telecine Machine"
                title = "Choose the telecine machine"
                choices = ["Flashtransfer", "Flashscan", "Scanner"]
                scanner = choicebox(msg, title, choices)
        
        if scanner == "Flashtransfer":
            def scanner(number):            
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Telecine', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', '16mm Film Digitisation', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'MWA', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Flashtransfer', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:settings[1]', fps_string, revtmd_xmlfile)
        if scanner == "Flashscan":
            def scanner(number):            
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Telecine', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', '8mm Film Digitisation', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'MWA', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Flashscan', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'SDI', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:settings[1]', fps_string, revtmd_xmlfile)
        if scanner == "Scanner":
            def scanner(number):            
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:role', 'Scanning', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:description', '35mm Film Digitisation', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:manufacturer', 'P&S Techniks', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:modelName', 'Steadyframe', revtmd_xmlfile)
                add_to_revtmd('//revtmd:codingProcessHistory' + str([number]) + '/revtmd:signal', 'Ethernet', revtmd_xmlfile)
                
                
        # Preparation History        
        msg ="Preperation?"
        title = "Workflows"
        choices = ["Splice and perforation assessment",
                   "Splice repairs",
                   "Leader added", 
                   "Perforation repairs",
                   "Recanned",
                   "Cleaning",
                   "Mould removal"]
        preparation = multchoicebox(msg, title, choices, preselect=None)
        print preparation
        # End preperation history
        # Flashtransfer settings  
        flashtransfer_settings = []   
        if scanner == "Flashtransfer": 
            msg ="Flashtransfer settings?"
            title = "Workflows"
            choices = ["Gamma Curve - Linear",
                       "Gamma Curve - Low",
                       "Gamma Curve - Standard", 
                       "Gamma Curve - High",
                       "Contrast - Low",
                       "Contrast - Off",
                       "Contrast - High",
                       "None",]
            flashtransfer_settings = multchoicebox(msg, title, choices, preselect=None)
            print flashtransfer_settings
            # End preperation history
        
        # Interventions during capture
        msg ="Interventions at point of capture"
        title = "capture interventions"
        choices = ["Exposure compensation",
                   "Contrast adjustment",
                   "Negative to positive", 
                   "Spacing between reels not captured",
                   "Monochrome setting enabled",
                   "Overscanning of image to capture optical track"
                    ]
        capture_interventions = multchoicebox(msg, title, choices, preselect=None)
        # End interventions during capture
        if not workflow == "Scanning":
            # Audio assesment during capture
            msg ="Headphones/speakers used during capture"
            title = "capture audio assesment"
            choices = ["Philips Headphones",
                       "M-Audio Speakers",
                       "Both",
                       "None"                   
                   
                        ]
            audio_choices = choicebox(msg, title, choices)

            if audio_choices == "Philips Headphones":
                def audio_capture_telecine(number):
                    philips_headphones_telecine(number)
            elif audio_choices == "M-Audio Speakers":
                def audio_capture_telecine(number):
                    maudio_left_speaker_telecine(number)
                    number += 1
                    maudio_right_speaker_telecine(number)

            elif audio_choices == "Both":
                def audio_capture_telecine(number):        
                    philips_headphones_telecine(number) 
                    number += 1
                
                    maudio_left_speaker_telecine(number)
                    number += 1
                    maudio_right_speaker_telecine(number)
            elif audio_choices == "None": 
                print "No audio selected"        
            print audio_choices       
              
        
        # End interventions during capture
        msg ="Post Processing?"
        title = "Post Processing"
        choices = ["Horizontal Flipping",
                   "Vertical Flipping",
                   "Audio normalised to -20db", 
                   "Broadcast Safe Filter",
                   "Desaturate Filter",
                   "Negative to positive",
                   "Avid Motion Editing",
                   "None"
                   ]
        post_processing = multchoicebox(msg, title, choices, preselect=None)

    #More interviews    
    msg ="User?"
    title = "Pick a name yo!"
    choices = ["Kieran O'Leary", "Gavin Martin", 
               "Dean Kavanagh", "Raelene Casey", 
           "Anja Mahler", "Eoin O'Donohoe", "Brian Cash","Unknown"]
    user = choicebox(msg, title, choices)

    msg = "Fill out these things please"
    title = "blablablabl"
    fieldNames = ["Source Accession Number",
                  "Filmographic Reference Number", 
                  "Identifier-Object Entry/Accession Number:"]
      # we start with blanks for the values
    fieldValues = multenterbox(msg,title, fieldNames)

    # make sure that none of the fields was left blank
    while 1:
            if fieldValues == None: break
            errmsg = ""
            for i in range(len(fieldNames)):
                if fieldValues[i].strip() == "":
                    errmsg = errmsg + ('"%s" is a required field.' % fieldNames[i])
            if errmsg == "": break # no problems found
            fieldValues = multenterbox(errmsg, title, fieldNames, fieldValues)
    print fieldValues  
    # Prints info to screen. Make this actually useful! 
    #print "Reply was:", fieldValues
    #print "Your selection was:\n Workflow =  %s\n Scanner = %s\n Preparation actions = %s\n User = %s\n" % (workflow,scanner, preparation, user)

    # Store md5 checksum.
    print 'generating md5 checksum, this may take some time'
    md5 = subprocess.check_output(['md5deep', '-e', sys.argv[1]])
    
    duration             = get_mediainfo('duration',
                                         '--inform=Video;%Duration_String4%', 
                                         sys.argv[1] )
    par                  = get_mediainfo('par',
                                         '--inform=Video;%PixelAspectRatio%',
                                         sys.argv[1] )
    dar                  = get_mediainfo('dar', 
                                         '--inform=Video;%DisplayAspectRatio%',
                                         sys.argv[1] )
    width                = get_mediainfo('width', 
                                         '--inform=Video;%Width%',
                                         sys.argv[1] )
    height               = get_mediainfo('height',
                                         '--inform=Video;%Height%',
                                         sys.argv[1] )
    vcodec               = get_mediainfo('dar',
                                         '--inform=Video;%Codec%',
                                         sys.argv[1] )
    size                 = get_mediainfo('size',
                                         '--inform=Video;%StreamSize%', 
                                         sys.argv[1] )
    chromasubsampling    = get_mediainfo('chromasubsampling',
                                         '--inform=Video;%ChromaSubsampling%', 
                                         sys.argv[1] )
    codec_id             = get_mediainfo('codecid', 
                                         '--inform=Video;%CodecID%',
                                         sys.argv[1] )
    scantype             = get_mediainfo('scantype',
                                         '--inform=Video;%ScanType%', 
                                         sys.argv[1] )
    scanorder            = get_mediainfo('scanorder', 
                                         '--inform=Video;%ScanOrder%', 
                                         sys.argv[1] )
    bitspersample        = get_mediainfo('bitspersample', 
                                         '--inform=Video;%BitDepth%', 
                                         sys.argv[1] )
    quality              = get_mediainfo('quality', 
                                         '--inform=Video;%Compression_Mode%',
                                         sys.argv[1] )
    container_duration   = get_mediainfo('container_duration',
                                         '--inform=General;%Duration_String4%',
                                         sys.argv[1] )
    container_size       = get_mediainfo('container_size',
                                         '--inform=General;%FileSize%', 
                                         sys.argv[1] )
    overall_bitrate      = get_mediainfo('overall_bitrate', 
                                         '--inform=General;%OverallBitRate%',
                                         sys.argv[1] )
    video_bitrate        = get_mediainfo('video_bitrate',
                                         '--inform=Video;%BitRate%',
                                         sys.argv[1] )
    container_mime       = get_mediainfo('container_mime',
                                         '--inform=General;%InternetMediaType%',
                                         sys.argv[1] )
    frame_rate           = get_mediainfo('frame_rate', 
                                         '--inform=Video;%FrameRate%',
                                         sys.argv[1] )
    container_frame_rate = get_mediainfo('container_frame_rate',
                                         '--inform=General;%FrameRate%',
                                         sys.argv[1] )
    container_name       = get_mediainfo('container_frame_rate',
                                         '--inform=General;%Format%',
                                         sys.argv[1] )
    container_profile    = get_mediainfo('container_profile',
                                         '--inform=General;%Format_Profile%',
                                         sys.argv[1] )
    frame_count          = get_mediainfo('frame_count', 
                                         '--inform=Video;%FrameCount%',
                                         sys.argv[1] )

    # Create mostly blank reVTMD template which we'll gradually fill up with info.
    with open(revtmd_xmlfile, "w+") as fo:

        fo.write('<?xml version="1.0" encoding="UTF-8"?>\n')
        fo.write('<revtmd xmlns:revtmd="http://nwtssite.nwts.nara/schema/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://nwtssite.nwts.nara/schema/  http://www.archives.gov/preservation/products/reVTMD.xsd">\n')
        fo.write('<revtmd:reVTMD>\n')
        fo.write('<revtmd:object>\n')
        fo.write('<revtmd:filename/>\n')
        fo.write('<revtmd:organization>\n')
        fo.write('<revtmd:organization_main>\n')
        fo.write('<revtmd:name>Irish Film Institute</revtmd:name>\n')
        fo.write('<revtmd:role/>\n')
        fo.write('</revtmd:organization_main>\n')
        fo.write('<revtmd:organization_division>\n')
        fo.write('<revtmd:name>Irish Film Archive</revtmd:name>\n')
        fo.write('</revtmd:organization_division>\n')
        fo.write('</revtmd:organization>\n')
        fo.write('<revtmd:identifier type="Object Entry">%s</revtmd:identifier>\n' % fieldValues[2])
        fo.write('<revtmd:identifier type="Inmagic DB Textworks Filmographic Reference Number">%s</revtmd:identifier>\n' % fieldValues[1])
        
        fo.write('<revtmd:duration>%s</revtmd:duration>\n' % container_duration)
        fo.write('<revtmd:language/>\n')
        fo.write('<revtmd:size>%s</revtmd:size>\n' % container_size)
        fo.write('<revtmd:datarate>%s</revtmd:datarate>\n' % overall_bitrate)
        fo.write('<revtmd:use>Preservation Master</revtmd:use>\n')
        fo.write('<revtmd:color/>\n')
        fo.write('<revtmd:framerate>%s</revtmd:framerate>\n' % container_frame_rate)
        fo.write('<revtmd:format>\n')
        fo.write('<revtmd:name>%s</revtmd:name>\n' % container_name)
        fo.write('<revtmd:mimetype>%s</revtmd:mimetype>\n' % container_mime)
        fo.write('<revtmd:version>%s</revtmd:version>\n' % container_profile)
        fo.write('</revtmd:format>\n')        
        fo.write('<!-- Checksum as generated immediately after the digitization process. -->\n')
        fo.write('<revtmd:checksum algorithm="md5" dateTime="%s">%s</revtmd:checksum>\n' % (time_date,md5.split()[0])) 
        fo.write('<revtmd:sound>%s</revtmd:sound>\n' % sound)
        fo.write('<revtmd:track id="1" type="video">\n')
        fo.write('<revtmd:duration/> \n')
        fo.write('<revtmd:size/>\n')
        fo.write('<revtmd:dataRate/>\n' )
        fo.write('<revtmd:color/>\n')
        fo.write('<revtmd:frame>\n')
        fo.write('<revtmd:pixelsHorizontal/>\n')
        fo.write('<revtmd:pixelsVertical/>\n')
        fo.write('<revtmd:PAR/>\n')
        fo.write('<revtmd:DAR/>\n')
        fo.write('</revtmd:frame>\n')
        fo.write('<revtmd:frameRate>%s</revtmd:frameRate>\n' % frame_rate)
        fo.write('<revtmd:codec>\n')
        fo.write('<revtmd:codecID/>\n')
        fo.write('<revtmd:channelCount/>\n')
        fo.write('<revtmd:endianness/>\n')
        fo.write('<revtmd:quality/>\n')
        fo.write('<revtmd:scanType/>\n')
        fo.write('<revtmd:scanOrder/>\n')
        fo.write('</revtmd:codec>\n')
        fo.write('<revtmd:bitsPerSample/>\n')
        fo.write('<revtmd:sampling/>\n')
        fo.write('<revtmd:frameCount>%s</revtmd:frameCount> \n' % frame_count)
        fo.write('</revtmd:track>\n')
        revtmd_blank_audio_fields()
        fo.write('<revtmd:captureHistory>\n')
        fo.write('<revtmd:digitizationDate>%s</revtmd:digitizationDate>\n' % date)
        fo.write('<revtmd:digitizationEngineer/>\n')
        fo.write('<revtmd:preparationActions/>\n')
        fo.write('<revtmd:preparationActions/>\n')
        fo.write('<revtmd:preparationActions/>\n')
        fo.write('<revtmd:preparationActions/>\n')
        fo.write('<revtmd:preparationActions/>\n')
        fo.write('<revtmd:preparationActions/>\n')
        fo.write('<revtmd:source/>\n')
        for _ in range(no_of_emptyfields):    
            revtmd_coding_process_history()
        fo.write('</revtmd:captureHistory>\n')
        fo.write('</revtmd:object>\n')
        fo.write('</revtmd:reVTMD>\n')
        fo.write('</revtmd>\n')

    prep1 = 1
    if preparation != None:

        for prep_actions in preparation: 
                 add_to_revtmd('//revtmd:preparationActions' + '[' + str(prep1) + ']' , prep_actions, revtmd_xmlfile)
                 prep1 += 1    

    # Combine previous functions for the bestlight workflow  
    counter = 0
    def steadyframe(position):
        add_to_revtmd('//revtmd:filename', filename_without_path, revtmd_xmlfile)
        add_to_revtmd('//revtmd:source', fieldValues[0], revtmd_xmlfile)
        position += 1
        basement_2k_scanner(position)
        position += 1
        scanner(position)
        position += 1
        basement_content_agent_pc(position)
        position += 1
        basement_content_agent_pc_os_revtmd(position)
        position += 1
        tech_metadata_revtmd()
        position += 1
        add_to_revtmd('//revtmd:digitizationEngineer[1]', user, revtmd_xmlfile)
        #ffmpeg_revtmd(position)        
        prep_data_entry()
        capture_interventions_func()
    def bestlight(position):
        add_to_revtmd('//revtmd:filename', filename_without_path, revtmd_xmlfile)
        add_to_revtmd('//revtmd:source', fieldValues[0], revtmd_xmlfile)
        position += 1
        bmd_us4k_revtmd(position)
        position += 1
        scanner(position)
        position += 1
        telecine_mac_pro_revtmd(position)
        position += 1
        telecine_mac_pro_os_revtmd(position)
        position += 1
        IiyamaMonitor1_telecine(position)
        position += 1
        IiyamaMonitor2_telecine(position)
        position += 1
        tvlogic_broadcast_telecine(position)
        position += 1
        if not audio_choices == "None":         
            audio_capture_telecine(position)
            
            if audio_choices == "Philips Headphones":
                position += 1
            elif audio_choices == "M-Audio Speakers":
                position += 2
            elif audio_choices == "Both":
                position += 3
        bmd_ultrascopes_telecine(position)
        tech_metadata_revtmd()
        position += 1
        avid_capture_revtmd(position)
        position += 1
        add_to_revtmd('//revtmd:digitizationEngineer[1]', user, revtmd_xmlfile)
        avid_consolidate_revtmd(position)
        position += 1
        avid_post_processing(position)
        position += 1
        avid_export_revtmd(position)
        #ffmpeg_revtmd(position)        
        prep_data_entry()
        capture_interventions_func()
               
    def ingest1():
        add_to_revtmd('//revtmd:filename', filename_without_path, revtmd_xmlfile)
        add_to_revtmd('//revtmd:source', fieldValues[0], revtmd_xmlfile)
        add_to_revtmd('//revtmd:digitizationEngineer[1]', user, revtmd_xmlfile)
        aja_analog2digital(4)
        tech_metadata_revtmd()
        win7_hp_revtmd(3)
        #ffmpeg_revtmd(6)
        workstation(2)
        control_room_capture_revtmd(5)
        deck_func(1)
        
    def ingest2():
        add_to_revtmd('//revtmd:filename', filename_without_path, revtmd_xmlfile)
        add_to_revtmd('//revtmd:source', fieldValues[0], revtmd_xmlfile)
        add_to_revtmd('//revtmd:digitizationEngineer[1]', user, revtmd_xmlfile)
        aja_analog2digital(4)
        tech_metadata_revtmd()
        win7_hp_revtmd(3)
        ffmpeg_revtmd(6)
        workstation(2)
        control_room_capture_revtmd(5)
        deck_func(1)

    # This launches the xml creation based on your selections  
    if workflow == "bestlight":
        bestlight(counter)
    elif workflow == "Tape Ingest 1":
        if deck not in ("UVW-1400AP", "UVW-1200P"):
            ingest1()
        else:
            def betasp_ingest1():
                add_to_revtmd('//revtmd:filename', filename_without_path, revtmd_xmlfile)
                add_to_revtmd('//revtmd:source', fieldValues[0], revtmd_xmlfile)
                add_to_revtmd('//revtmd:digitizationEngineer[1]', user, revtmd_xmlfile)
                aja_kona_capture_ingest1(5)
                tech_metadata_revtmd()
                win7_hp_revtmd(3)
                workstation(2)
                control_room_capture_revtmd(6)
                deck_func(1)
                bmd_miniconverter_revtmd(4)
            betasp_ingest1()
            
    elif workflow == "Tape Ingest 2":
        ingest2()
    elif workflow == "Scanning":
        steadyframe(counter)

    # Temporary, possibly permanent way to delete empty elements
    subprocess.call(['xml', 'ed', '--inplace','-d',
                    '//*[not(./*) and (not(./text()) or normalize-space(./text())="")]',
                     revtmd_xmlfile])



#!/usr/bin/env python
import subprocess
import sys
import os
from glob import glob


def get_audio_stream_count(filename):
	audio_stream_count = subprocess.check_output(['ffprobe', '-v', 'error', '-select_streams', 'a', '-show_entries', 'stream=index','-of', 'flat', filename]).splitlines()
	return len(audio_stream_count)


def make_qctools(input):
    qctools_args = ['ffprobe', '-f', 'lavfi', '-i',]
    audio_tracks = get_audio_stream_count(input)
    if audio_tracks > 0:
        qctools_args += ["movie=%s:s=v+a[in0][in1],[in0]signalstats=stat=tout+vrep+brng,cropdetect=reset=1:round=1,split[a][b];[a]field=top[a1];[b]field=bottom[b1],[a1][b1]psnr[out0];[in1]ebur128=metadata=1,astats=metadata=1:reset=1:length=0.4[out1]" % input]
    elif audio_tracks == 0:
        qctools_args += ["movie=%s,signalstats=stat=tout+vrep+brng,cropdetect=reset=1,split[a][b];[a]field=top[a1];[b]field=bottom[b1],[a1][b1]psnr" % input]
    qctools_args += ['-show_frames', '-show_versions', '-of', 'xml=x=1:q=1', '-noprivate']
    print qctools_args
    qctoolsreport = subprocess.check_output(qctools_args)
    return qctoolsreport

def write_qctools_gz(qctoolsxml, sourcefile):
    with open(qctoolsxml, "w+") as fo:
        fo.write(make_qctools(sourcefile))
    subprocess.call(['gzip', qctoolsxml])

input = sys.argv[1]
print input

# Store the directory containing the input file/directory.
wd = os.path.dirname(input)

# Change current working directory to the value stored as "wd"
os.chdir(os.path.abspath(wd))
print os.path.abspath(wd), 111

# Store the actual file/directory name without the full path.
file_without_path = os.path.basename(input)
print file_without_path

# Check if input is a file.
# AFAIK, os.path.isfile only works if full path isn't present.
if os.path.isfile(file_without_path):
    print os.path.isfile(file_without_path)
    print "single file found"
    video_files = []                       # Create empty list
    video_files.append(file_without_path)  # Add filename to list
    print video_files
    for i in video_files:
        qctoolsxml_file = i + '.qctools.xml'
        write_qctools_gz(qctoolsxml_file, i)

# Check if input is a directory.
elif os.path.isdir(file_without_path):
    for root, dirs, filenames in os.walk(sys.argv[1]):
        for filename in filenames:
            os.chdir(root)
            if filename.endswith(('.mkv', '.mov')):
                qctoolsxml_file = filename + '.qctools.xml'
                write_qctools_gz(qctoolsxml_file, filename)


# Prints some stuff if input isn't a file or directory.
else:
    print "Your input isn't a file or a directory."
    print "What was it? I'm curious."






#!/usr/bin/env python3
'''
Generates SIPS by calling various microservices and functions.


add in a check at the start - check if AF number is actually in the CSV.
'''
import os
import argparse
import sys
import shutil
import datetime
import time
import copyit
import ififuncs
import package_update
import accession
import manifest
import makezip
import accession
from masscopy import analyze_log
try:
    from clairmeta.utils.xml import prettyprint_xml
    from clairmeta import DCP
    import dicttoxml
except ImportError:
    print('Clairmeta is not installed. DCP options will not function!')


def make_folder_path(path, args, object_entry):
    '''
    Generates objects/logs/metadata/UUID folder structure in output directory.
    Asks user for UUID if it's not supplied in an arg.
    Adds a workaround for special collections workflows.
    Returns the path.
    UNITTEST - does path exist
    '''
    if not args.u:
        representation_uuid = ififuncs.create_uuid()
    else:
        representation_uuid = args.u
    if args.sc:
        oe_path = args.o
    else:
        oe_path = os.path.join(path, object_entry)
    path = os.path.join(oe_path, representation_uuid)
    ififuncs.make_folder_structure(path)
    return path


def consolidate_manifests(path, directory, new_log_textfile):
    '''
    Consolidates all manifests in the objects folder
    moves old manifests into logs
    renames manifest with uuid and updates paths in manifest textfile.
    '''
    uuid = os.path.basename(path)
    objects_dir = os.path.join(path, directory)
    new_manifest_textfile = os.path.join(
        os.path.dirname(path), uuid + '_manifest.md5'
    )
    collective_manifest = []
    for manifest in os.listdir(objects_dir):
        if ififuncs.check_if_manifest(manifest):
            ififuncs.generate_log(
                new_log_textfile,
                'EVENT = Manifest consolidation - Checksums from %s merged into %s' % (os.path.join(objects_dir, manifest), new_manifest_textfile)
            )
            with open(os.path.join(objects_dir, manifest), 'r', encoding='utf-8') as fo:
                manifest_lines = fo.readlines()
                for i in manifest_lines:
                    # This is what appends the new path to existing paths.
                    new_manifest_path = uuid + '/%s/' % directory + i[34:]
                    collective_manifest.append(
                        i[:32] + '  ' + new_manifest_path
                    )
            # Cut and paste old manifests into the log directory
            shutil.move(
                objects_dir + '/' +  manifest, os.path.join(path, 'logs')
            )
            ififuncs.generate_log(
                new_log_textfile,
                'EVENT = Manifest movement - Manifest from %s to %s' % (objects_dir + '/' +  manifest, os.path.join(path, 'logs'))
            )
    with open(new_manifest_textfile, 'a', encoding='utf-8') as manifest_object:
        for checksums in collective_manifest:
            manifest_object.write(checksums)
    return new_manifest_textfile


def consolidate_logs(lognames, path):
    '''
    Finds moveit.py logs on the desktop
    Copies all text into a single log file
    Saves it in the SIP
    '''
    uuid = os.path.basename(path)
    new_log_textfile = os.path.join(path, 'logs' + '/' + uuid + '_sip_log.log')
    for log in lognames:
        with open(log, 'r', encoding='utf-8') as fo:
            log_lines = fo.readlines()
        with open(new_log_textfile, 'a', encoding='utf-8') as log_object:
            for lines in log_lines:
                log_object.write(lines)


def move_files(inputs, sip_path, args, user):
    '''
    Runs moveit.py on all inputs
    '''
    log_names = []
    for item in inputs:
        cmd = [item, os.path.join(sip_path, 'objects')]
        if args.move:
            cmd.append('-move')
        if args.l:
            cmd.append('-l')
        log_name = copyit.main(cmd)
        log_names.append(log_name)
        if args.rename_uuid:
            if os.path.isfile(item):
                objects_dir = os.path.join(sip_path, 'objects')
                uuid = os.path.basename(sip_path)
                old_basename, ext = os.path.splitext(item)
                new_path = os.path.join(objects_dir, uuid + ext)
                os.rename(os.path.join(objects_dir, os.path.basename(item)), new_path)
                manifest = os.path.join(os.path.dirname(new_path), os.path.basename(item)) + '_manifest.md5'
                updated_lines = []
                ififuncs.generate_log(
                    log_name,
                    'EVENT = Filename change - eventDetail=original filename replaced with uuid, eventOutcomeDetailNote=%s replaced with %s, agentName=%s, agentName=sipcreator.py))' % (os.path.basename(item), uuid + ext, user))
                with open(manifest, 'r') as file_object:
                    checksums = file_object.readlines()
                    for line in checksums:
                        if os.path.basename(item) in line:
                            line = line.replace(os.path.basename(item), os.path.basename(new_path))
                            updated_lines.append(line)
                with open(manifest, 'w') as fo:
                    for lines in updated_lines:
                        fo.write(lines)
    consolidate_logs(log_names, sip_path)
    return log_names


def log_report(log_names):
    '''
    Analyzes all the moveit.py logs on the desktop and print a report.
    '''
    desktop_logs_dir = ififuncs.make_desktop_logs_dir()
    for i in log_names:
        if os.path.isfile(i):
            print(("%-*s   :  copyit job was a %s" % (50, os.path.basename(i)[:-24], analyze_log(i))))
        else:
            print((i, 'can\'t find log file, trying again...'))
            log_names.remove(i)
            for logs in os.listdir(desktop_logs_dir):
                # look at log filename minus the seconds and '.log'
                if os.path.basename(i)[:-7] in logs:
                    # make sure that the alternate log filename is more recent
                    if int(
                            os.path.basename(logs)[-12:-4].replace('_', '')
                    ) > int(
                        os.path.basename(i)[-12:-4].replace('_', '')):
                        print(('trying to analyze %s' % logs))
                        print(("%-*s   : %s" % (
                            50, os.path.basename(logs)[:-24], analyze_log(
                                os.path.join(desktop_logs_dir, logs))
                            )))
                        log_names.append(os.path.join(desktop_logs_dir, logs))

def parse_args(args_):
    '''
    Parse command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Wraps objects into an Irish Film Institute SIP'
        ' Written by Kieran O\'Leary.'
    )
    parser.add_argument(
        '-i', nargs='+',
        help='full path of input directory', required=True
    )
    parser.add_argument(
        '-o', '-output',
        help='full path of output directory', required=True
    )
    parser.add_argument(
        '-u', '-uuid',
        help='Use a pre-existing UUID instead of a newly generated UUID.'
    )
    parser.add_argument(
        '-rename_uuid', action='store_true',
        help='Use with caution! This will rename an object with a randonly generated UUID'
    )
    parser.add_argument(
        '-user',
        help='Declare who you are. If this is not set, you will be prompted.'
    )
    parser.add_argument(
        '-d', '-dcp', action='store_true',
        help='Adds DCP specific processing, like creating objects subfolder with text extracted from <ContentTitleText> in the CPL.'
    )
    parser.add_argument(
        '-quiet', action='store_true',
        help='Quiet mode, suppresses the analyze_logs() report'
    )
    parser.add_argument(
        '-move', action='store_true',
        help='invokes the -move argument in copyit.py - moves instead of copy.'
    )
    parser.add_argument(
        '-l', action='store_true',
        help='invokes the -lto argument in copyit.py - uses gcp instead of rsync.'
    )
    parser.add_argument(
        '-sc', action='store_true',
        help='special collections workflow'
    )
    parser.add_argument(
        '-zip', action='store_true',
        help='Uses makezip.py to store the objects in an uncompressed ZIP'
    )
    parser.add_argument(
        '-accession', action='store_true',
        help='Launches accession.py immediately after sipcreator.py finishes. This is only useful if the SIP has already passed QC and will definitely be accessioned and ingested.'
    )
    parser.add_argument(
        '-filmo_csv',
        help='Enter the path to the Filmographic CSV so that the metadata will be stored within the package.'
    )
    parser.add_argument(
        '-oe',
        help='Enter the Object Entry number for the representation.SIP will be placed in a folder with this name.'
    )
    parser.add_argument(
        '-manifest',
        help='Enter the full path to a manifest for the files within a ZIP. This will be stored in the supplemental folder.'
    )
    parser.add_argument(
        '-supplement', nargs='+',
        help='Enter the full path of files or folders that are to be added to the supplemental subfolder within the metadata folder. Use this for information that supplements your preservation objects but is not to be included in the objects folder.'
    )
    parsed_args = parser.parse_args(args_)
    return parsed_args


def create_content_title_text(sip_path, args):
    '''
    DCPs are often delivered with inconsistent foldernames.
    This will rename the parent folder with the value recorded in <ContentTitleText>
    For example:
    Original name: CHARBON-SMPTE-24
    New name: CHARBON-SMPTE-24-INTEROP-SUBS_TST_S_XX-EN_FR_XX_2K_CHA-20120613_CHA_OV
    Rename will only occur if user agrees.
    '''
    cpl = ififuncs.find_cpl(args.i[0])
    objects_dir = os.path.join(sip_path, 'objects')
    dcp_dirname = os.path.dirname(cpl)
    content_title = ififuncs.get_contenttitletext(cpl)
    dci_foldername = os.path.join(objects_dir, content_title)
    rename_dcp = ififuncs.ask_yes_no(
        'Do you want to rename %s with %s ?' % (os.path.basename(dcp_dirname), dci_foldername)
    )
    if rename_dcp == 'N':
        print('Exiting')
        sys.exit()
    return content_title

def normalise_objects_manifest(sip_path):
    '''
    For a root copy workflow, the objects manifest is in the
    uuid directory, not the objects directory. This will move it
    into the objects directory.
    '''
    objects_manifest = os.path.join(sip_path, 'objects_manifest.md5')
    if os.path.isfile(objects_manifest):
        updated_manifest_lines = []
        with open(objects_manifest, 'r') as fo:
            manifest_lines = fo.readlines()
            for i in manifest_lines:
                # This is what appends the new path to existing paths.
                replacement = i.replace('  objects/', '  ')
                updated_manifest_lines.append(replacement)
        with open(objects_manifest, 'w') as fo:
            for x in updated_manifest_lines:
                fo.write(x)
        # Cut and paste old manifests into the log directory
        shutil.move(
            objects_manifest, os.path.join(sip_path, 'objects')
        )
def get_object_entry(args):
    '''
    Figures out which OE number to use and performs some basic validation.
    UNITTEST - use the existing ifs to perform some True/False tests.
    '''
    if not args.sc:
        if args.oe:
            if args.oe[:2] != 'oe':
                print('First two characters must be \'oe\' and last four characters must be four digits')
                object_entry = ififuncs.get_object_entry()
            elif len(args.oe[2:]) not in list(range(4, 6)):
                print('First two characters must be \'oe\' and last four characters must be four digits')
                object_entry = ififuncs.get_object_entry()
            elif not args.oe[2:].isdigit():
                object_entry = ififuncs.get_object_entry()
                print('First two characters must be \'oe\' and last four characters must be four digits')
            else:
                object_entry = args.oe
        else:
            object_entry = ififuncs.get_object_entry()
    else:
        object_entry = 'not_applicable'
    return object_entry


def determine_uuid(args, sip_path):
    '''
    Validates a UUID to use as the SIP identifier.
    UNITTEST = validate the existing validations.
    '''
    if args.u:
        if ififuncs.validate_uuid4(args.u) is None:
            uuid = args.u
            uuid_event = (
                'EVENT = eventType=Identifier assignement,'
                ' eventIdentifierType=UUID, value=%s, module=uuid.uuid4'
            ) % uuid
        else:
            print('exiting due to invalid UUID')
            sys.exit()
    else:
        uuid = os.path.basename(sip_path)
        uuid_event = (
            'EVENT = eventType=Identifier assignement,'
            ' eventIdentifierType=UUID, value=%s, module=uuid.uuid4'
        ) % uuid
    return uuid, uuid_event

def process_dcp(sip_path, content_title, args, new_manifest_textfile, new_log_textfile, metadata_dir, clairmeta_version):
    '''
    Runs DCP specific functions.
    '''
    objects_dir = os.path.join(sip_path, 'objects')
    cpl = ififuncs.find_cpl(objects_dir)
    dcp_dirname = os.path.dirname(cpl)
    os.chdir(os.path.dirname(dcp_dirname))
    os.rename(os.path.basename(dcp_dirname), content_title)
    new_dcp_path = os.path.join('objects', content_title).replace("\\", "/")
    absolute_dcp_path = os.path.join(sip_path, new_dcp_path)
    ififuncs.manifest_replace(
        new_manifest_textfile,
        os.path.join('objects', os.path.basename(args.i[0])).replace("\\", "/"),
        new_dcp_path
    )
    '''
    a = subprocess.check_output(['python', '-m', 'clairmeta.cli', 'check', '-type', 'dcp', absolute_dcp_path], stderr=subprocess.STDOUT)
    b = subprocess.check_output(['python', '-m', 'clairmeta.cli', 'probe', '-type', 'dcp', '-format', 'xml', absolute_dcp_path], stderr=subprocess.STDOUT)
    '''
    dcp = DCP(absolute_dcp_path)
    dcp_dict = dcp.parse()
    # json_str = json.dumps(dcp_dict , sort_keys=True, indent=2, separators=(',', ': '))
    xml_str = dicttoxml.dicttoxml(dcp_dict, custom_root='ClairmetaProbe', ids=False, attr_type=False)
    xml_pretty = prettyprint_xml(xml_str)
    status, report = dcp.check()
    ififuncs.generate_log(
        new_log_textfile,
        'EVENT = eventType=validation, eventOutcome=%s, eventDetail=%s, agentName=Clairmeta version %s' % (
            status, report, clairmeta_version
        )
    )
    clairmeta_xml = os.path.join(metadata_dir, '%s_clairmeta.xml' % content_title)
    ififuncs.generate_log(
        new_log_textfile,
        'EVENT = Metadata extraction - eventDetail=Clairmeta DCP metadata extraction, eventOutcome=%s, agentName=Clairmeta version %s' % (clairmeta_xml, clairmeta_version)
    )
    with open(clairmeta_xml, 'w') as fo:
        fo.write(xml_pretty)
    ififuncs.checksum_replace(new_manifest_textfile, new_log_textfile, 'md5')
    ififuncs.manifest_update(new_manifest_textfile, clairmeta_xml)
    print(status)
    print(report)

def make_oe_register():
    '''
    This sends a placeholder oe register to the desktop logs directory.
    This should get rid of some of the more painful, repetitive identifier matching.
    '''
    desktop_logs_dir = ififuncs.make_desktop_logs_dir()
    oe_register = os.path.join(
        desktop_logs_dir,
        'oe_helper_register_' + time.strftime("%Y-%m-%dT%H_%M_%S.csv")
    )
    ififuncs.create_csv(oe_register, (
        'OE No.',
        'Date Received',
        'Quantity',
        'Format',
        'Description',
        'Contact Name',
        'Type of Acquisition',
        'Accession No.',
        'Additional Information',
        'Habitat',
        'Vinegar No'
    ))
    return oe_register
def main(args_):
    '''
    Launch all the functions for creating an IFI SIP.
    '''
    args = parse_args(args_)
    start = datetime.datetime.now()
    inputs = args.i
    for input in inputs:
        if ififuncs.check_av_or_doc(input) == 'av':
            ififuncs.check_existence(['mediainfo'])
        elif ififuncs.check_av_or_doc(input) == 'doc':
            ififuncs.check_existence(['sf', 'exiftool'])
    if args.d:
        try:
            import clairmeta
            clairmeta_version = clairmeta.__version__
        except ImportError:
            print('Exiting as Clairmeta is not installed. If there is a case for not using clairmeta, please let me know and i can make a workaround')
            sys.exit()
    if args.zip:
        ififuncs.check_existence(['7za'])
    print(args)
    user = ififuncs.determine_user(args)
    object_entry = get_object_entry(args)
    sip_path = make_folder_path(os.path.join(args.o), args, object_entry)
    uuid, uuid_event = determine_uuid(args, sip_path)
    new_log_textfile = os.path.join(sip_path, 'logs' + '/' + uuid + '_sip_log.log')
    if args.d:
        content_title = create_content_title_text(sip_path, args)
    ififuncs.generate_log(
        new_log_textfile,
        'EVENT = sipcreator.py started'
    )
    ififuncs.generate_log(
        new_log_textfile,
        'eventDetail=sipcreator.py %s' % ififuncs.get_script_version('sipcreator.py')
    )
    ififuncs.generate_log(
        new_log_textfile,
        'Command line arguments: %s' % args
    )
    ififuncs.generate_log(
        new_log_textfile,
        'EVENT = agentName=%s' % user
    )
    ififuncs.generate_log(
        new_log_textfile,
        uuid_event
    )
    if not args.sc:
        ififuncs.generate_log(
            new_log_textfile,
            'EVENT = eventType=Identifier assignement,'
            ' eventIdentifierType=object entry, value=%s'
            % object_entry
        )
    metadata_dir = os.path.join(sip_path, 'metadata')
    supplemental_dir = os.path.join(metadata_dir, 'supplemental')
    logs_dir = os.path.join(sip_path, 'logs')
    if args.accession:
        accession_number = ififuncs.get_accession_number()
        reference_number = ififuncs.get_reference_number()
        parent = ififuncs.ask_question('What is the parent record? eg MV 1234. Enter n/a if this is a born digital acquisition with no parent.')
        donor = ififuncs.ask_question('Who is the source of acquisition, as appears on the donor agreement? This will not affect Reproductions.')
        reproduction_creator = ififuncs.ask_question('Who is the reproduction creator? This will not affect acquisitions. Enter n/a if not applicable')
        depositor_reference = ififuncs.ask_question('What is the donor/depositor number? This will not affect Reproductions.')
        acquisition_type = ififuncs.get_acquisition_type('')
        donation_date = ififuncs.ask_question('When was the donation date in DD/MM/YYYY format? Eg. 31/12/1999 - Unfortunately this is NOT using ISO 8601.')
    if args.zip:
        inputxml, inputtracexml, dfxml = ififuncs.generate_mediainfo_xmls(inputs[0], args.o, uuid, new_log_textfile)
        if args.manifest:
            shutil.copy(args.manifest, args.manifest.replace('_manifest.md5', '_manifest-md5.txt'))
            source_manifest = args.manifest.replace('_manifest.md5', '_manifest-md5.txt')
        else:
            source_manifest = os.path.join(
                args.o,
                os.path.basename(args.i[0]) + '_manifest-md5.txt'
            )
            ififuncs.generate_log(
                new_log_textfile,
                'EVENT = message digest calculation, status=started, eventType=messageDigestCalculation, agentName=hashlib, eventDetail=MD5 checksum of source files within ZIP'
            )
            ififuncs.hashlib_manifest(args.i[0], source_manifest, os.path.dirname(args.i[0]))
            ififuncs.generate_log(
                new_log_textfile,
                'EVENT = message digest calculation, status=finished, eventType=messageDigestCalculation, agentName=hashlib, eventDetail=MD5 checksum of source files within ZIP'
            )
        ififuncs.generate_log(
            new_log_textfile,
            'EVENT = packing, status=started, eventType=packing, agentName=makezip.py, eventDetail=Source object to be packed=%s' % inputs[0]
        )
        makezip_judgement, zip_file = makezip.main(['-i', inputs[0], '-o', os.path.join(sip_path, 'objects'), '-basename', uuid + '.zip'])
        ififuncs.generate_log(
            new_log_textfile,
            'EVENT = packing, status=finished, eventType=packing, agentName=makezip.py, eventDetail=Source object packed into=%s' % zip_file
        )
        if makezip_judgement is None:
            judgement = 'lossless'
        else:
            judgement = makezip_judgement
        ififuncs.generate_log(
            new_log_textfile,
            'EVENT = losslessness verification, status=finished, eventType=messageDigestCalculation, agentName=makezip.py, eventDetail=embedded crc32 checksum validation, eventOutcome=%s' % judgement
        )
        ififuncs.generate_log(
            new_log_textfile,
            'EVENT = losslessness verification, status=finished, eventType=messageDigestCalculation, agentName=makezip.py, eventDetail=embedded crc32 checksum validation, eventOutcome=%s' % judgement
        )
    else:
        log_names = move_files(inputs, sip_path, args, user)
    ififuncs.get_technical_metadata(sip_path, new_log_textfile)
    ififuncs.hashlib_manifest(
        metadata_dir, metadata_dir + '/metadata_manifest.md5', metadata_dir
    )
    if args.sc:
        normalise_objects_manifest(sip_path)
    new_manifest_textfile = consolidate_manifests(sip_path, 'objects', new_log_textfile)
    if args.zip:
        if zip_file.endswith('.001'):
            for split_archive in os.listdir(os.path.dirname(zip_file)):
                ififuncs.generate_log(
                    new_log_textfile, 'EVENT = Message Digest Calculation, status=started, eventType=message digest calculation, eventDetail=%s module=hashlib' % split_archive
                )
                ififuncs.manifest_update(new_manifest_textfile, os.path.join(os.path.dirname(zip_file), split_archive))
                ififuncs.generate_log(
                    new_log_textfile, 'EVENT = Message Digest Calculation, status=finished, eventType=message digest calculation, eventDetail=%s module=hashlib' % split_archive
                )
        else:
            ififuncs.generate_log(
                new_log_textfile, 'EVENT = Message Digest Calculation, status=started, eventType=message digest calculation, eventDetail=%s module=hashlib' % zip_file
            )
            ififuncs.manifest_update(new_manifest_textfile, zip_file)
            ififuncs.generate_log(
                new_log_textfile, 'EVENT = Message Digest Calculation, status=finished, eventType=message digest calculation, eventDetail=%s module=hashlib' % zip_file
            )
    consolidate_manifests(sip_path, 'metadata', new_log_textfile)
    ififuncs.hashlib_append(
        logs_dir, new_manifest_textfile,
        os.path.dirname(os.path.dirname(logs_dir))
    )
    if args.supplement:
        os.makedirs(supplemental_dir)
        supplement_cmd = ['-i', args.supplement, '-user', user, '-new_folder', supplemental_dir, os.path.dirname(sip_path), '-copy']
        package_update.main(supplement_cmd)
    if args.zip:
        os.makedirs(supplemental_dir)
        supplement_cmd = ['-i', [inputxml, inputtracexml, dfxml, source_manifest], '-user', user, '-new_folder', supplemental_dir, os.path.dirname(sip_path), '-copy']
        package_update.main(supplement_cmd)
    if args.sc:
        print('Generating Digital Forensics XML')
        dfxml = accession.make_dfxml(args, sip_path, uuid)
        ififuncs.generate_log(
            new_log_textfile,
            'EVENT = Metadata extraction - eventDetail=File system metadata extraction using Digital Forensics XML, eventOutcome=%s, agentName=makedfxml' % (dfxml)
        )
        ififuncs.manifest_update(new_manifest_textfile, dfxml)
        sha512_log = manifest.main([sip_path, '-sha512', '-s'])
        sha512_manifest = os.path.join(
            os.path.dirname(sip_path), uuid + '_manifest-sha512.txt'
        )
        ififuncs.merge_logs_append(sha512_log, new_log_textfile, new_manifest_textfile)
        ififuncs.checksum_replace(sha512_manifest, new_log_textfile, 'sha512')
        os.remove(sha512_log)
    ififuncs.sort_manifest(new_manifest_textfile)
    if not args.quiet:
        if 'log_names' in locals():
            log_report(log_names)
    finish = datetime.datetime.now()
    print('\n- %s ran this script at %s and it finished at %s' % (user, start, finish))
    if args.d:
        process_dcp(sip_path, content_title, args, new_manifest_textfile, new_log_textfile, metadata_dir, clairmeta_version)
    if args.accession:
        register = accession.make_register()
        filmographic_dict = ififuncs.extract_metadata(args.filmo_csv)[0]
        for filmographic_record in filmographic_dict:
            if filmographic_record['Reference Number'].lower() == reference_number.lower():
                if filmographic_record['Title'] == '':
                    title = filmographic_record['TitleSeries'] + '; ' + filmographic_record['EpisodeNo']
                else:
                    title = filmographic_record['Title']
        oe_register = make_oe_register()
        ififuncs.append_csv(oe_register, (object_entry.upper()[:2] + '-' + object_entry[2:], donation_date, '1','',title,donor,acquisition_type[1], accession_number, 'Representation of %s|Reproduction of %s' % (reference_number, parent), ''))
        accession_cmd = [
            os.path.dirname(sip_path), '-user', user,
            '-force',
            '-number', accession_number,
            '-reference', reference_number,
            '-register', register,
            '-filmo_csv', args.filmo_csv,
            '-pbcore'
        ]
        if not parent.lower() == 'n/a':
            accession_cmd.extend(['-parent', parent])
        accession_cmd.extend(['-donor', donor])
        accession_cmd.extend(['-depositor_reference', depositor_reference])
        accession_cmd.extend(['-acquisition_type', acquisition_type[2]])
        accession_cmd.extend(['-donation_date', donation_date])
        accession_cmd.extend(['-reproduction_creator', reproduction_creator])
        print(accession_cmd)
        accession.main(accession_cmd)
    return new_log_textfile, new_manifest_textfile


if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3

'''
Usage: seq2ffv1.py source_parent_directory output_directory
The script will fnd a single reel, multi-reel (with or without sidecar WAV)
image sequence beneath the source_parent_directory.
The script will then:
* Create a folder structure for each image sequence in
your designated output_directory.
* Transcode to a single FFV1 in Matroska file, or multiple Matroska files
for multi-reel films.
* Verify losslessness by fully reversing the FFV1/MKV file and validating
the source checksums.
'''
import subprocess
import os
import argparse
import tempfile
import shutil
import time
import sys
import ififuncs
import sipcreator
import makezip
import deletefiles

def short_test(images):
    '''
    Perform a test on the first 24 frames that will encode via Rawcooked,
    then decode back to the original form,
    and the whole file checksums of the original 24 frames
    and the restored 24 frames are compared.
    '''
    temp_uuid = ififuncs.create_uuid()
    temp_dir = os.path.join(tempfile.gettempdir(), temp_uuid)
    os.makedirs(temp_dir)
    # Only analyse the first 24 frames.
    for image in sorted(os.listdir(images))[:24]:
        full_path = os.path.join(images, image)
        shutil.copy(full_path, temp_dir)
    mkv_uuid = ififuncs.create_uuid()
    mkv_file = os.path.join(tempfile.gettempdir(), mkv_uuid + '.mkv')
    subprocess.call(['rawcooked', temp_dir, '-c:a', 'copy', '-o', mkv_file])
    converted_manifest = os.path.join(temp_dir, '123.md5')
    ififuncs.hashlib_manifest(temp_dir, converted_manifest, temp_dir)
    subprocess.call(['rawcooked', mkv_file])
    rawcooked_dir = mkv_file + '.RAWcooked'
    restored_dir = os.path.join(rawcooked_dir, temp_uuid)
    restored_manifest = os.path.join(restored_dir, '456.md5')
    ififuncs.hashlib_manifest(restored_dir, restored_manifest, restored_dir)
    judgement = ififuncs.diff_textfiles(converted_manifest, restored_manifest)
    print((' - Deleting temp directory %s' % temp_dir))
    try:
        shutil.rmtree(temp_dir)
    except WindowsError:
        print('Sorry, we do not have permission to delete these files')
    print((' - Deleting temp reversibility directory %s' % rawcooked_dir))
    try:
        shutil.rmtree(rawcooked_dir)
    except WindowsError:
        print('Sorry, we do not have permission to delete these files')
    print((' - Deleting temp FFV1/MKV %s' % mkv_file))
    os.remove(mkv_file)
    return judgement

def reversibility_verification(objects, source_manifest, reversibility_dir):
    '''
    Restore the MKV back to DPX, create a checksum, and compare to source DPX
    checksums.
    Return a value of lossy or lossless.
    '''
    temp_uuid = ififuncs.create_uuid()
    temp_dir = os.path.join(reversibility_dir, temp_uuid)
    os.makedirs(temp_dir)
    for ffv1_mkv in objects:
        subprocess.call(['rawcooked', ffv1_mkv, '-o', temp_dir])
    converted_manifest = os.path.join(temp_dir, '123.md5')
    ififuncs.hashlib_manifest(temp_dir, converted_manifest, temp_dir)
    judgement = ififuncs.diff_textfiles(converted_manifest, source_manifest)
    print((' - Deleting temp directory %s' % temp_dir))
    try:
        shutil.rmtree(temp_dir)
    except WindowsError:
        print('Unable to delete temp directory, sorry!')
    return judgement

def run_loop(args):
    '''
    This will only process one sequence. Batch processing will come later.
    '''
    current_dir = os.path.dirname(os.path.abspath(sys.argv[0]))
    if args.user:
        user = args.user
    else:
        user = ififuncs.get_user()
    object_entry = ififuncs.get_object_entry()
    log_name_source = os.path.join(
        args.o, '%s_seq2ffv1_log.log' % time.strftime("_%Y_%m_%dT%H_%M_%S")
    )
    ififuncs.generate_log(log_name_source, 'seq2ffv1.py started.')
    ififuncs.generate_log(
        log_name_source,
        'eventDetail=seq2ffv1.py %s' % ififuncs.get_script_version('seq2ffv1.py'))
    ififuncs.generate_log(
        log_name_source,
        'Command line arguments: %s' % args
    )
    ififuncs.generate_log(
        log_name_source,
        'EVENT = agentName=%s' % user
    )
    uuid = ififuncs.create_uuid()
    verdicts = []
    multi_reeler = False
    source_directory = args.i
    images = ififuncs.get_image_sequence_files(source_directory)
    if images == 'none':
        print('no images found in directory - checking for multi-reel sequence')
        images = ififuncs.check_multi_reel(source_directory)
        multi_reeler = True
        if images == 'none':
            sys.exit()
    # this is checking for a single reeler.
    else:
        images = [source_directory]
    reel_number = 1
    objects = []
    short_test_reports = []
    rawcooked_logfiles = []
    for reel in images:
        short_test_reports.append(short_test(reel))
        for i in short_test_reports:
            print((' - 24 frame reversibility test for %s is %s' % (os.path.basename(reel), i)))
            if i == 'lossy':
                print('It appears that this sequence is not reversible - exiting')
                sys.exit()
        time.sleep(2)
        # check for a/b rolls
        if reel[-1] in ['a', 'b']:
            reel_number = reel[-2]
        ffv1_path, source_abspath, args, log_name_source, normalisation_tool, rawcooked_logfile = make_ffv1(
            reel,
            args,
            log_name_source,
            reel_number,
            uuid,
            multi_reeler
        )
        objects.append(ffv1_path)
        rawcooked_logfiles.append(rawcooked_logfile)
        # check for a/b rolls
        if not reel[-1] in ['a', 'b']:
            reel_number += 1
    judgement = package(objects, object_entry, uuid, source_abspath, args, log_name_source, normalisation_tool, user, rawcooked_logfiles, multi_reeler, current_dir)
    judgement, sipcreator_log, sipcreator_manifest = judgement
    verdicts.append([source_directory, judgement])
    for verdict in verdicts:
        print(("%-*s   : %s" % (50, args.i, verdict[1])))
    ififuncs.generate_log(log_name_source, 'seq2ffv1.py finished.')
    ififuncs.merge_logs(log_name_source, sipcreator_log, sipcreator_manifest)


def verify_losslessness(source_textfile, ffv1_md5):
    '''
    Compares two framemd5 documents in order to determine losslessness.
    '''
    judgement = ififuncs.diff_textfiles(source_textfile, ffv1_md5)
    checksum_mismatches = []
    with open(source_textfile) as source_md5_object:
        with open(ffv1_md5) as ffv1_md5_object:
            for (line1), (line2) in zip(
                    ififuncs.read_lines(
                        source_md5_object
                    ), ififuncs.read_lines(ffv1_md5_object)
            ):
                if line1 != line2:
                    if 'sar' in line1:
                        checksum_mismatches = ['sar']
                    else:
                        checksum_mismatches.append(1)
    return judgement

def make_ffv1(
        reel,
        args,
        log_name_source,
        reel_number,
        uuid,
        multi_reeler
    ):
    '''
    This launches the image sequence to FFV1/Matroska process
    as well as framemd5 losslessness verification.
    '''
    output_dirname = args.o
    if multi_reeler:
        if reel[-1] in ['a', 'b']:    
            mkv_basename = uuid + '_reel%s%s.mkv' % (str(reel_number), reel[-1])
        else:
            mkv_basename = uuid + '_reel%s.mkv' % str(reel_number)
    else:
        mkv_basename = uuid + '.mkv'
    ffv1_path = os.path.join(output_dirname, mkv_basename)
    rawcooked_logfile = os.path.join(
        args.o, '%s_rawcooked.log' % mkv_basename
    )
    normalisation_tool = ififuncs.get_rawcooked_version()
    rawcooked_logfile = "\'" + rawcooked_logfile + "\'"
    env_dict = ififuncs.set_environment(rawcooked_logfile)
    rawcooked_cmd = ['rawcooked', reel, '--check', 'full', '-c:a', 'copy', '-o', ffv1_path]
    if args.framerate:
        rawcooked_cmd.extend(['-framerate', args.framerate])
    ffv12dpx = (rawcooked_cmd)
    print(ffv12dpx)
    if args.zip:
        uuid = ififuncs.create_uuid()
        # ugly hack until i recfactor. this is the zip_path, not ffv1_path
        ffv1_path = os.path.join(output_dirname, uuid + '.zip')
        ififuncs.generate_log(
            log_name_source,
            'EVENT = packing, status=started, eventType=packing, agentName=makezip.py, eventDetail=Source object to be packed=%s' % reel
        )
        makezip_judgement = makezip.main([
            '-i', reel,
            '-o', output_dirname,
            '-basename', os.path.basename(ffv1_path)
        ])[0]
        ififuncs.generate_log(
            log_name_source,
            'EVENT = packing, status=finished, eventType=packing, agentName=makezip.py, Source object packed into=%s' % ffv1_path
        )
        if makezip_judgement is None:
            judgement = 'lossless'
        else:
            judgement = makezip_judgement
        ififuncs.generate_log(
            log_name_source,
            'EVENT = losslessness verification, status=finished, eventType=messageDigestCalculation, agentName=makezip.py, eventDetail=embedded crc32 checksum validation, eventOutcome=%s' % judgement
        )
    if not args.zip:
        ififuncs.generate_log(
            log_name_source,
            'EVENT = normalisation, status=started, eventType=Creation, agentName=%s, eventDetail=Image sequence normalised to FFV1 in a Matroska container'
            % normalisation_tool
        )
        subprocess.call(ffv12dpx, env=env_dict)
        ififuncs.generate_log(
            log_name_source,
            'EVENT = normalisation, status=finshed, eventType=Creation, agentName=%s, eventDetail=Image sequence normalised to FFV1 in a Matroska container'
            % normalisation_tool
        )
    return ffv1_path, reel, args, log_name_source, normalisation_tool, rawcooked_logfile

def package(objects, object_entry, uuid, source_abspath, args, log_name_source, normalisation_tool, user, rawcooked_logfiles, multi_reeler, current_dir):
    '''
    Package the MKV using sipcreator.py
    '''
    sip_dir = os.path.join(
        args.o, os.path.join(object_entry, uuid)
    )
    inputxml, inputtracexml, dfxml = ififuncs.generate_mediainfo_xmls(source_abspath, args.o, uuid, log_name_source)
    source_manifest = os.path.join(
        args.o,
        os.path.basename(args.i) + '_manifest-md5.txt'
    )
    ififuncs.generate_log(
        log_name_source,
        'EVENT = message digest calculation, status=started, eventType=messageDigestCalculation, agentName=hashlib, eventDetail=MD5 checksum of source files'
    )
    if multi_reeler:
        ififuncs.hashlib_manifest(args.i, source_manifest, args.i)
    else:
        ififuncs.hashlib_manifest(args.i, source_manifest, os.path.dirname(args.i))
    ififuncs.generate_log(
        log_name_source,
        'EVENT = message digest calculation, status=finished, eventType=messageDigestCalculation, agentName=hashlib, eventDetail=MD5 checksum of source files'
    )
    ififuncs.generate_log(
        log_name_source,
        'EVENT = losslessness verification, status=started, eventType=messageDigestCalculation, agentName=%s, eventDetail=Full reversibility of %s back to its original form, followed by checksum verification using %s ' % (normalisation_tool, objects, source_manifest)
    )
    if args.reversibility_dir:
        reversibility_dir = args.reversibility_dir
    else:
        reversibility_dir = args.o

    judgement = reversibility_verification(objects, source_manifest, reversibility_dir)
    ififuncs.generate_log(
        log_name_source,
        'EVENT = losslessness verification, status=finished, eventType=messageDigestCalculation, agentName=%s, eventDetail=Full reversibilty of %s back to its original form, followed by checksum verification using %s , eventOutcome=%s' % (normalisation_tool, objects, source_manifest, judgement)
    )
    supplement_cmd = ['-supplement', inputxml, inputtracexml, dfxml, source_manifest]
    if args.supplement:
        supplement_cmd.extend(args.supplement)
    sipcreator_cmd = [
        '-i',
    ]
    for i in objects:
        sipcreator_cmd.append(i)
    sipcreator_cmd += [
        '-u',
        uuid,
        '-quiet',
        '-move',
        '-user',
        user,
        '-oe',
        object_entry,
        '-o', args.o
    ]
    sipcreator_cmd.extend(supplement_cmd)
    sipcreator_log, sipcreator_manifest = sipcreator.main(sipcreator_cmd)
    logs_dir = os.path.join(sip_dir, 'logs')
    for files in os.listdir(logs_dir):
        if files.endswith('.md5'):
            deletefiles.main(['-i', os.path.join(logs_dir, files), '-uuid_path', sip_dir, '-user', user])
    for rawcooked_logfile in rawcooked_logfiles:
        rawcooked_logfile = rawcooked_logfile.replace('\'', '')
        shutil.move(rawcooked_logfile, logs_dir)
        ififuncs.manifest_update(
            sipcreator_manifest,
            os.path.join(logs_dir, os.path.basename(rawcooked_logfile))
        )
    metadata_dir = os.path.join(sip_dir, 'metadata')
    os.chdir(current_dir)
    shutil.copy(os.path.join(os.path.abspath(os.path.dirname(sys.argv[0])), 'film_scan_aip_documentation.txt'), metadata_dir)
    ififuncs.manifest_update(
        sipcreator_manifest,
        os.path.join(metadata_dir, 'film_scan_aip_documentation.txt')
    )
    os.remove(dfxml)
    os.remove(inputtracexml)
    os.remove(inputxml)
    return judgement, sipcreator_log, sipcreator_manifest

def setup():
    '''
    Sets up a lot of the variables and filepaths.
    '''
    parser = argparse.ArgumentParser(description='Transcode all DPX or TIFF'
                                     ' image sequence in the subfolders of your'
                                     ' source directory to FFV1 Version 3'
                                     ' in a Matroska Container.'
                                     ' Written by Kieran O\'Leary.')
    parser.add_argument(
        '-i',
        help='Input directory'
    )
    parser.add_argument(
        '-o',
        help='Destination directory'
    )
    parser.add_argument(
        '-user',
        help='Declare who you are. If this is not set, you will be prompted.')
    parser.add_argument(
        '-reversibility_dir',
        help='This argument requires the full path of the location that you want to use for the reversibility directory. By default, seq2ffv1 will use your output dir for storing the temporary reversibility files.')
    parser.add_argument(
        '-framerate',
        help='This argument triggers the -framerate option in rawcooked and allows you to override the default fps value, eg -framerate 16')
    parser.add_argument(
        '-zip',
        help='Use makezip.py to generate an uncompressed zip file', action='store_true'
    )
    parser.add_argument(
        '-supplement', nargs='+',
        help='Enter the full path of files or folders that are to be added to the supplemental subfolder within the metadata folder. Use this for information that supplements your preservation objects but is not to be included in the objects folder.'
    )
    args = parser.parse_args()
    return args

def main():
    '''
    Overly long main function that does most of the heavy lifting.
    '''
    ififuncs.check_existence(['rawcooked', 'ffmpeg'])
    args = setup()
    run_loop(args)

if __name__ == '__main__':
    main()
